/*
 *
 *   TEA - A quick and simple text preprocessor
 *   Copyright (C) 2010-2016 DarkBatcher
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include <stdio.h>
#include <stdlib.h>

#include <libDos9.h>
#include "tea_out.h"
#include "../Tea.h"

void Tea_HtmlOutputHandler(TEAPAGE* lpTeaPage, FILE* pFile, int i, char** argv)
{
	ESTR *lpEsStr=Dos9_EsInit(),
		 *lpEsTitle=Dos9_EsInit(),
		 *lpEsMeta=Dos9_EsInit(),
		 *lpEsHead=Dos9_EsInit(),
		 *lpEsFoot=Dos9_EsInit();

	char *lpHeadTag="<h1>",
	      *lpHeadTagEnd="</h1>";

	int iLastLevel=0,
	    iCompMode=0,
	    iBckLevel;

	/* execute command line */


	while (argv[i]) {

		if (!stricmp(argv[i], "/T")) {

			if (!argv[++i]) {

				fputs("TEA :: Error : Invalid parameter (after /T)\n", stderr);
				goto error;

			}

			Dos9_EsCpy(lpEsTitle, argv[i]);

		} else if (!strcmp(argv[i], "/H")) {

			if (!argv[++i]) {

				fputs("TEA :: Error : Invalid parameter (after /H)\n", stderr);
				goto error;

			}

			Tea_HtmlLoadFile(lpEsHead, argv[i]);


		} else if (!strcmp(argv[i], "/F")) {

			if (!argv[++i]) {

				fputs("TEA :: Error : Invalid parameter (after /F)\n", stderr);
				goto error;

			}

			Tea_HtmlLoadFile(lpEsFoot, argv[i]);

		} else if (!strcmp(argv[i], "/M")) {

			if (!argv[++i]) {

				fputs("TEA :: Error : Invalid parameter (after /M)\n", stderr);
				goto error;

			}

			Tea_HtmlLoadFile(lpEsMeta, argv[i]);

		} else if (!strnicmp(argv[i], "/C", 2)) {

			argv[i]+=2;

			if (*(argv[i])==':')
				argv[i]++;

			if (!stricmp(argv[i], "HTML4")) {

				iCompMode=1;

			} else if (!stricmp(argv[i], "XHTML")) {

				iCompMode=2;

			} else {

				fprintf(stderr,"TEA :: Error : ``%s'' is not a valid html flavour.\n", argv[i]);
				goto error;

			}

		} else {

			fprintf(stderr,"TEA :: Error : Invalid argument \"%s\"\n", argv[i]);
			goto error;

		}

		i++;
	}


	/* Get the first heading to put it on the
	   the title of the document */
	Tea_HtmlGetTitle(lpTeaPage, lpEsStr);
	Dos9_EsCatE(lpEsTitle, lpEsStr);

	if (iCompMode==0) {

		fprintf(pFile, "<!DOCTYPE html>\n"
		        "<!--\n"
		        "\tThis HTML5 compliant page was generated by the\n"
		        "\tTEA text processor (version 2.0)\n\n"
		        "\tLearn more or get involved at http://www.dos9.org\n"
		        "-->\n"
		        "<html>\n"
		        "<head>\n"
		        "<meta charset=\"%s\" />\n"
		        "<title>%s</title>\n"
		        "%s\n"
		        "</head>\n"
		        "<body>\n"
		        "%s\n",
		        Dos9_EsToChar(lpEsEncoding),
		        Dos9_EsToChar(lpEsTitle),
		        Dos9_EsToChar(lpEsMeta),
		        Dos9_EsToChar(lpEsHead));

	} else if (iCompMode==1) {

		fprintf(pFile, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3c.org/TR/html4/strict.dtd\">\n"
		        "<!--\n"
		        "\tThis HTML 4.01 compliant page was generated by the\n"
		        "\tTEA text processor (version 2.0)\n\n"
		        "\tLearn more or get involved at http://www.dos9.org\n"
		        "-->\n"
		        "<html>\n"
		        "<head>\n"
		        "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\" />\n"
		        "<title>%s</title>\n"
		        "%s\n"
		        "</head>\n"
		        "<body>\n"
		        "%s\n",
		        Dos9_EsToChar(lpEsEncoding),
		        Dos9_EsToChar(lpEsTitle),
		        Dos9_EsToChar(lpEsMeta),
		        Dos9_EsToChar(lpEsHead));


	} else if (iCompMode==2) {

		fprintf(pFile, "<!DOCTYPE html PUBLIC \"-//W3C//DTD//XHTML 1.0 Strict//EN\" \"http://www.w3c.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n"
		        "<!--\n"
		        "\tThis xHTML 1.1 compliant page was generated by the\n"
		        "\tTEA text processor (version 2.0)\n\n"
		        "\tLearn more or get involved at http://www.dos9.org\n"
		        "-->\n"
		        "<html xmlns=\"http://www.w3c.org/1999/xhtml\">\n"
		        "<head>\n"
		        "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\" />\n"
		        "<title>%s</title>\n"
		        "%s\n"
		        "</head>\n"
		        "<body>\n"
		        "%s\n",
		        Dos9_EsToChar(lpEsEncoding),
		        Dos9_EsToChar(lpEsTitle),
		        Dos9_EsToChar(lpEsMeta),
		        Dos9_EsToChar(lpEsHead));


	}

	while (lpTeaPage) {

		switch (lpTeaPage->iBlockType) {

		case TEA_BLOCK_HEADING:

			while (iLastLevel) {

				fputs("</li></ul>\n", pFile);
				iLastLevel--;

			}

			fputs(lpHeadTag, pFile);

			Dos9_EsCpy(lpEsStr, lpTeaPage->lpBlockContent);
			Dos9_EsReplace(lpEsStr, "\n", "<br />");
			fputs(Dos9_EsToChar(lpEsStr), pFile);


			fputs(lpHeadTagEnd, pFile);

			lpHeadTag="<h2>";
			lpHeadTagEnd="</h2>";

			break;

		case TEA_BLOCK_CODE:

			while (iLastLevel > lpTeaPage->iBlockLevel) {

				fputs("</li></ul>\n", pFile);
				iLastLevel--;

			}

			while (iLastLevel < lpTeaPage->iBlockLevel) {

				fputs("<ul><li>\n", pFile);
				iLastLevel++;

			}

			fputs("<code>", pFile);

			Dos9_EsCpy(lpEsStr, lpTeaPage->lpBlockContent);
			Dos9_EsReplace(lpEsStr, "\n", "<br />");
			fputs(Dos9_EsToChar(lpEsStr), pFile);

			fputs("</code>", pFile);

			break;

		case TEA_BLOCK_PARAGRAPH:

			iBckLevel=iLastLevel;

			while (iLastLevel > lpTeaPage->iBlockLevel) {

				fputs("</li></ul>\n", pFile);
				iLastLevel--;

			}

			while (iLastLevel < lpTeaPage->iBlockLevel) {

				fputs("<ul><li>\n", pFile);
				iLastLevel++;

			}

			if (!(iLastLevel>iBckLevel)
			    && !(lpTeaPage->iFlag & TEA_LIST_NO_MARK)
			    && iLastLevel)
				fputs("</li><li>", pFile);



			fputs("<p>", pFile);

			Tea_HtmlOutputParagraph(lpTeaPage->lpTeaNode, pFile);

			fputs("</p>", pFile);


		}

		fputs("\n\n", pFile);

		lpTeaPage=lpTeaPage->lpTeaNext;

	}

	while (iLastLevel) {

		fputs("</li></ul>\n", pFile);
		iLastLevel--;

	}

	fprintf(pFile, "%s\n"
	        "</body>\n"
	        "</html>",
	        Dos9_EsToChar(lpEsFoot));


	Dos9_EsFree(lpEsStr);
	Dos9_EsFree(lpEsTitle);
	Dos9_EsFree(lpEsHead);
	Dos9_EsFree(lpEsMeta);
	Dos9_EsFree(lpEsFoot);
	return;

error:
	Dos9_EsFree(lpEsStr);
	Dos9_EsFree(lpEsTitle);
	Dos9_EsFree(lpEsHead);
	Dos9_EsFree(lpEsMeta);
	Dos9_EsFree(lpEsFoot);
	exit(-1);
}

void Tea_HtmlOutputParagraph(TEANODE* lpTeaNode, FILE* pFile)
{
	char* lpCh, *ext;
	int   bFirstLine=TRUE;

	while (lpTeaNode) {

		if (lpTeaNode->iNodeType==TEA_NODE_LINK) {

            if (!(ext = strrchr(lpTeaNode->lpTarget, '/')))
                ext = lpTeaNode->lpTarget;

            if (!(ext = strchr(ext, '.'))) {

                ext = ".html";

            } else {

                ext = "";

            }

            if (!strncmp(lpTeaNode->lpTarget, "http://", 7)
                || !strncmp(lpTeaNode->lpTarget, "https://", 8)
                || !strncmp(lpTeaNode->lpTarget, "mail://", 7))
                ext = "";

			fprintf(pFile, "<a href=\"%s%s\">", lpTeaNode->lpTarget, ext);

		} else if (lpTeaNode->iNodeType==TEA_NODE_EMPHASIS) {

			fputs("<strong>", pFile);

		}

		lpCh=lpTeaNode->lpContent;

		if ((lpTeaNode->iNodeType==TEA_NODE_PLAIN_TEXT)
		    && (bFirstLine)
		    && (*lpCh=='-')) {

			lpCh++;

		}

		bFirstLine=FALSE;

		fputs(lpCh, pFile);

		if (lpTeaNode->iNodeType==TEA_NODE_LINK) {

			fputs("</a>", pFile);

		} else if (lpTeaNode->iNodeType==TEA_NODE_EMPHASIS) {

			fputs("</strong>", pFile);

		}


		lpTeaNode=lpTeaNode->lpTeaNodeNext;

	}

}

void Tea_HtmlParseHandler(int iMsg ,void* lpData)
{

	ESTR* lpEsStr;

	if (iMsg!=TEA_MSG_READ_FILE)
		return;
	/* do nothing */

	lpEsStr=(ESTR*)lpData;

	Tea_HtmlEscapeChar(lpEsStr);
}

void Tea_HtmlEscapeChar(ESTR* lpEsStr)
{
	Dos9_EsReplace(lpEsStr, "&", "&amp;");
	Dos9_EsReplace(lpEsStr, "<", "&lt;");
	Dos9_EsReplace(lpEsStr, ">", "&gt;");

}

void Tea_HtmlGetTitle(TEAPAGE* lpTeaPage, ESTR* lpEsStr)
{

	while (lpTeaPage) {

		if (lpTeaPage->iBlockType == TEA_BLOCK_HEADING) {

			Dos9_EsCpy(lpEsStr, lpTeaPage->lpBlockContent);
			return;

		}

		lpTeaPage=lpTeaPage->lpTeaNext;

	}

}

void Tea_HtmlLoadFile(ESTR* lpEsReturn, char* lpFileName)
{

	ESTR* lpEsTmp=Dos9_EsInit();
	FILE* pFile;

	Dos9_EsCpy(lpEsReturn, "");

	if (!(pFile=fopen(lpFileName, "r"))) {

		fprintf(stderr, "TEA :: Error : Unable to load file \"%s\"\n", lpFileName);
		exit(-1);

	}

	while (!Dos9_EsGet(lpEsTmp,pFile)) {

		Dos9_EsCatE(lpEsReturn, lpEsTmp);

	}


	Dos9_EsFree(lpEsTmp);

}
